import matplotlib.pyplot as plt
import numpy as np
import scipy.stats

muA = 100       #   普通
muB = 120       #   適性あり
sgm = 10        #   標準偏差

x_values = np.arange(70, 150, 0.1)
ScoreA = scipy.stats.norm.pdf(x_values, muA, sgm)
ScoreB = scipy.stats.norm.pdf(x_values, muB, sgm)
plt.plot(x_values, ScoreA, label = 'Class A')
plt.plot(x_values, ScoreB, label = 'Class B')

def Lfunc(theta, x):
    if theta == 'A':
        return scipy.stats.norm.pdf(x, muA, sgm)
    elif theta == 'B':
        return scipy.stats.norm.pdf(x, muB, sgm)
    else:
        raise Exception('Invalid parameter value!')

LE = []
for x in x_values:
    LE.append(0 if Lfunc('A', x) > Lfunc('B', x) else 0.04)

plt.plot(x_values, LE, label = 'MLE')
plt.yticks([])
plt.legend(loc = 'upper left')
plt.title('Distributions and MLE', fontsize = 16)
plt.show()

prior = {'A': 0.5, 'B': 0.5}

def postP(theta, x, prior):
    vA = scipy.stats.norm.pdf(x, muA, sgm) * prior['A']
    vB = scipy.stats.norm.pdf(x, muB, sgm) * prior['B']
    if theta == 'A':
        return vA / (vA + vB)
    elif theta == 'B':
        return vB / (vA + vB)
    else:
        raise Exception('Invalid parameter value!')

PostA = []
PostB = []
for x in x_values:
    PostA.append(postP('A', x, prior))
    PostB.append(postP('B', x, prior))

plt.plot(x_values, PostA, color = 'b', label = 'P(A|Score)')
plt.plot(x_values, PostB, color = 'g', label = 'P(B|Score)')
plt.vlines(110, 0, 1, color = 'y', linewidth = 2, label = 'Score = 110')
plt.xlabel('Score')
plt.legend()
plt.title('Posterior Distributions at Scores\n' +\
          'Prior Distribution = [0.5, 0.5]', fontsize = 16)
plt.show()

prior = {'A': 0.99, 'B': 0.01}
print(prior['A'])

PostA = []
PostB = []
for x in x_values:
    PostA.append(postP('A', x, prior))
    PostB.append(postP('B', x, prior))

plt.plot(x_values, PostA, color = 'b', label = 'P(A|Score)')
plt.plot(x_values, PostB, color = 'g', label = 'P(B|Score)')
plt.vlines(110, 0, 1, color = 'y', linewidth = 2, label = 'Score = 110')
plt.xlabel('Score')
plt.legend()
plt.title('Posterior Distributions at Scores\n' +\
          'Prior Distribution = [0.99, 0.01]', fontsize = 16)
plt.show()


sgm = 2   #       標準偏差を２に設定

ScoreA = scipy.stats.norm.pdf(x_values, muA, sgm)
ScoreB = scipy.stats.norm.pdf(x_values, muB, sgm)
plt.plot(x_values, ScoreA, label = 'Class A')
plt.plot(x_values, ScoreB, label = 'Class B')

def Lfunc(theta, x):
    if theta == 'A':
        return scipy.stats.norm.pdf(x, muA, sgm)
    elif theta == 'B':
        return scipy.stats.norm.pdf(x, muB, sgm)
    else:
        raise Exception('Invalid parameter value!')

LE = []
for x in x_values:
    LE.append(0 if Lfunc('A', x) > Lfunc('B', x) else ScoreB.max())

plt.plot(x_values, LE, label = 'MLE')
plt.yticks([])
plt.legend(loc = 'upper left')
plt.title('Distributions and MLE\n' +\
          'sgm = 2,  Prior = [0.99, 0.01]', fontsize = 16)
plt.show()

def postP(theta, x, prior):
    vA = scipy.stats.norm.pdf(x, muA, sgm) * prior['A']
    vB = scipy.stats.norm.pdf(x, muB, sgm) * prior['B']
    if theta == 'A':
        return vA / (vA + vB)
    elif theta == 'B':
        return vB / (vA + vB)
    else:
        raise Exception('Invalid parameter value!')

PostA = []
PostB = []
for x in x_values:
    PostA.append(postP('A', x, prior))
    PostB.append(postP('B', x, prior))

plt.plot(x_values, PostA, color = 'b', label = 'P(A|Score)')
plt.plot(x_values, PostB, color = 'g', label = 'P(B|Score)')
plt.vlines(110, 0, 1, color = 'y', linewidth = 2, label = 'Score = 110')
plt.xlabel('Score')
plt.legend()
plt.title('Posterior Distributions at Scores\n' +\
          'sgm = 2,  Prior Distribution = [0.99, 0.01]', fontsize = 16)
plt.show()

print(prior)
